@@ -16,15 +16,22 @@
 ## 02111-1307, USA.
 
 markers = "+o*.xsd^v><";
+cmap=colormap("jet");
 
 clf();
 title(sprintf("%d Marker Styles",length(markers)));
 for i=1:length(markers),
-  plot(1:11,[1:11]+2*i,sprintf("-%s;Marker %s;",markers(i),markers(i)));
+  h=plot(1:11,[1:11]+2*i,sprintf("-%s;Marker %s;",markers(i),markers(i)));
+  
+  ## set the face color 
+  set(h,"MarkerFaceColor",cmap(i*3,:));
+  ## set the linewidth
+  set(h,"linewidth",mod(i,3)+1);
   hold on
 endfor
 hold off
 grid("on");
 axis([0 15 0 35]);
 
+printf("type line_props for help on line properties\n");
 
