@@ -1,4 +1,4 @@
-## Copyright (C) 2004 Shai Ayal
+# Copyright (C) 2004 Shai Ayal
 ##
 ## This program is free software; you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
@@ -26,17 +26,13 @@
 
 function h=area (x,y)
 
-  ca = gca();
-  if (isempty(ca))
-    cla();
-  elseif (strcmp(get(ca,"nextplot"),"replace")),
-    cla();
-  endif
-  
+  __nxtplt__();
+
+	h = [];
   for i=2:length(x),
-    h=patch([x(i-1),x(i-1),x(i),x(i)],\
+    h = [h, patch([x(i-1),x(i-1),x(i),x(i)],\
 	  [0     ,y(i-1),y(i),0   ],
-	  "LineStyle","none");
+	  "LineStyle","none")];
   endfor
 
 endfunction
