@@ -44,7 +44,7 @@
 #include "axes.h"
 
 
-#define OBJ_NAME "Axes"
+#define OBJ_NAME "axes"
 
 Axes::Axes(ocpl::Handle Parent) : Object(Parent) , numticks(6)
 {
@@ -53,9 +53,9 @@ Axes::Axes(ocpl::Handle Parent) : Object(Parent) , numticks(6)
   Properties["YLim"]     = Real2Matrix(0.0,1.0);
   Properties["ZLim"]     = Real2Matrix(-1.0,1.0);
   Properties["CLim"]     = Real2Matrix(0.0,1.0);
-  
+
   Text* txt;
-  
+
   txt = new Text(Self ,"",0,0,0);
   Properties["XLabel"]	 = new HandleVect(txt->GetHandle());
   Properties["XLabel"]->readonly(true);
@@ -70,7 +70,7 @@ Axes::Axes(ocpl::Handle Parent) : Object(Parent) , numticks(6)
   Properties["Title"]->readonly(true);
 
   Properties["NextPlot"] = new Radio("|add|{replace}|");
-  
+
   const char* automan = "|{auto}|manual|";
   Properties["XLimMode"] = new Radio(automan);
   Properties["YLimMode"] = new Radio(automan);
@@ -110,78 +110,73 @@ Axes::Axes(ocpl::Handle Parent) : Object(Parent) , numticks(6)
   SET_TYPE;
   COPY_DEFAULT(Position,Matrix);
   COPY_DEFAULT(ColorOrder,Matrix);
-  COPY_DEFAULT(Color,Color);
-  COPY_DEFAULT(XColor,Color);
-  COPY_DEFAULT(YColor,Color);
-  COPY_DEFAULT(ZColor,Color);
+  COPY_DEFAULT(Color,ColorNone);
+  COPY_DEFAULT(XColor,ColorNone);
+  COPY_DEFAULT(YColor,ColorNone);
+  COPY_DEFAULT(ZColor,ColorNone);
+  COPY_DEFAULT(XAxisLocation,Radio);
+  COPY_DEFAULT(YAxisLocation,Radio);
   COPY_DEFAULT(LineWidth,Scalar);
   COPY_DEFAULT(TickLength,Matrix);
   COPY_DEFAULT(XScale,Radio);
   COPY_DEFAULT(YScale,Radio);
   COPY_DEFAULT(ZScale,Radio);
   COPY_DEFAULT(GridLineStyle,Radio);
+  COPY_DEFAULT(Box,Radio);
 
-  SetupLimits();
-  //  AutoTicks();
+//   auto_tickers = new Ticks*[2];
+//   auto_tickers[0] = new Ticks(0,Self,10.);
+//   auto_tickers[1] = new Ticks(1,Self,10.);
+}
+
+Axes::~Axes()
+{
+  _DEBUG_PRINT("In ~axes\n");
+//   delete auto_tickers[0];
+//   delete auto_tickers[1];
+//   delete [] auto_tickers;
 }
 
 void Axes::Parse(ocpl::command& command)
 {
-  MAKE_REF(children,HandleVect);
+  //MAKE_REF(children,HandleVect);
   if(command.id()==ocpl::cla) {
-    children.Clear(); 
-
-    delete Properties["Legend"];
-    Legend* lgnd = new Legend(Self);
-    Properties["Legend"] = new HandleVect(lgnd->GetHandle());
-    Properties["Legend"]->readonly(true);
-
-    delete Properties["XLim"];
-    Properties["XLim"] = Real2Matrix(0,1);
-
-    delete Properties["YLim"];
-    Properties["YLim"] = Real2Matrix(0,1);
-
-    MAKE_REF(xlimmode,Radio);
-    MAKE_REF(ylimmode,Radio);
-    xlimmode.val("auto");
-    ylimmode.val("auto");
-    AutoTicks();
-    dirty = true;
+    Reset();
+    command.dirty(true);
     command.init_argout(0);
   }
   else if(command.id()==ocpl::line 
-	  || 
-	  command.id()==ocpl::patch
-	  ||
-	  command.id()==ocpl::surface
-	  ) {
+      || 
+      command.id()==ocpl::patch
+      ||
+      command.id()==ocpl::surface
+      ) {
     command.parsed(true);
 
     if(command.id()==ocpl::surface) {
       if(command.nargin() != 3) {
-	ret_error(command,"surface accepts 3 arguments");
-	return;
+        ret_error(command,"surface accepts 3 arguments");
+        return;
       }
     }
     else {
       if(command.nargin()!=2) {
-	ret_error(command,"line/patch accepts 2 arguments");
-	return;
+        ret_error(command,"line/patch accepts 2 arguments");
+        return;
       }
     }
 
-    Matrix XData(command.argin(0)->real_data(),
-		 command.argin(0)->nr,
-		 command.argin(0)->nc);
-    Matrix YData(command.argin(1)->real_data(),
-		 command.argin(1)->nr,
-		 command.argin(1)->nc);
-    
+    Matrix XData( command.argin(0)->real_data(),
+        command.argin(0)->nr,
+        command.argin(0)->nc);
+    Matrix YData( command.argin(1)->real_data(),
+        command.argin(1)->nr,
+        command.argin(1)->nc);
+
     // the data should not be deleted by command
     command.argin(0)->owner=false;
     command.argin(1)->owner=false;
-    
+
     double* hnd;
     if(command.id()==ocpl::line) {
       Line* line = new Line(GetHandle(),XData,YData);
@@ -195,20 +190,18 @@ void Axes::Parse(ocpl::command& command)
     }
     else if(command.id()==ocpl::surface) {
       Matrix ZData(command.argin(2)->real_data(),
-		   command.argin(2)->nr,
-		   command.argin(2)->nc);
+          command.argin(2)->nr,
+          command.argin(2)->nc);
       // the data should not be deleted by command
       command.argin(2)->owner=false;
-    
+
       Surface* surf = new Surface(GetHandle(),XData,YData,ZData);
       ((HandleVect *)GetProperty("Children"))->Add(surf->GetHandle());
       hnd = new double(static_cast<double>(surf->GetHandle()));
     }
-    
-    Axes::AutoScale();
-    Axes::AutoTicks();
+
     command.dirty(true);
-    
+
     //return handle
     command.argout(0,ocpl::real,1,1,reinterpret_cast<char*>(hnd),true);
   }
@@ -219,8 +212,8 @@ void Axes::Parse(ocpl::command& command)
       return;
     }
     if(command.argin(0)->id != ocpl::real 
-       || 
-       command.argin(0)->nr*command.argin(0)->nc < 2) {
+        || 
+        command.argin(0)->nr*command.argin(0)->nc < 2) {
       ret_error(command,"text: first argument must be position");
       return;
     }
@@ -228,19 +221,19 @@ void Axes::Parse(ocpl::command& command)
       ret_error(command,"text: second argument must be string");
       return;
     }
-    
+
     double *coord = command.argin(0)->real_data();
     Text* txt = 
       new Text(
-	       GetHandle(),
-	       command.argin(1)->data ,
-	       coord[0],
-	       coord[1],
-	       command.argin(0)->nr*command.argin(0)->nc > 2 ? coord[2] : 1 );
-    
+          GetHandle(),
+          command.argin(1)->data ,
+          coord[0],
+          coord[1],
+          command.argin(0)->nr*command.argin(0)->nc > 2 ? coord[2] : 1 );
+
     ((HandleVect *)GetProperty("Children"))->Add(txt->GetHandle());
     command.dirty(true);
-    
+
     //return handle
     double* hnd = new double(static_cast<double>(txt->GetHandle()));
     command.argout(0,ocpl::real,1,1,reinterpret_cast<char*>(hnd),true);
@@ -250,97 +243,114 @@ void Axes::Parse(ocpl::command& command)
 void Axes::PostSet(ocpl::command& com)
 {
   std::string prop(tolower(com.argin(1)->data));
-  
-  if(prop=="xlimmode" || 
-     prop=="ylimmode" || 
-     prop=="zlimmode" ||
-     prop=="xticklabelmode" ||
-     prop=="yticklabelmode" ||
-     prop=="zticklabelmode" ||
-     prop=="xtickmode" ||
-     prop=="ytickmode" ||
-     prop=="ztickmode" ||
-     prop=="xscale"   ||
-     prop=="yscale"   ||
-     prop=="zscale")
-  {
-    AutoScale();
-    AutoTicks();
-  }
-  else if(prop=="xlim") {
+
+  if(prop=="xlim") {
     MAKE_REF(xlimmode,Radio);
     xlimmode.val("manual");
-    AutoTicks();
   }
   else if(prop=="ylim") { 
     MAKE_REF(ylimmode,Radio);
     ylimmode.val("manual");	
-    AutoTicks();
   }	
   else if(prop=="zlim") {
     MAKE_REF(zlimmode,Radio);
     zlimmode.val("manual");
-    AutoTicks();
   }
   else if(prop=="xtick") {
     MAKE_REF(xtickmode,Radio);
     xtickmode.val("manual");
-    AutoTicks();
   }
   else if (prop=="ytick") {
     MAKE_REF(ytickmode,Radio);
     ytickmode.val("manual");
-    AutoTicks();
   }
   else if(prop=="ztick") {
     MAKE_REF(ztickmode,Radio);
     ztickmode.val("manual");
-    AutoTicks();
   }
   else if(prop=="xticklabel") {
+    MAKE_REF(xtickmode,Radio);
+    xtickmode.val("manual");
     MAKE_REF(xticklabelmode,Radio);
     xticklabelmode.val("manual");
-    AutoTicks();
   }
   else if(prop=="yticklabel") {
+    MAKE_REF(ytickmode,Radio);
+    ytickmode.val("manual");
     MAKE_REF(yticklabelmode,Radio);
     yticklabelmode.val("manual");
-    AutoTicks();
   }
   else if(prop=="zticklabel") {
+    MAKE_REF(ztickmode,Radio);
+    ztickmode.val("manual");
     MAKE_REF(zticklabelmode,Radio);
     zticklabelmode.val("manual");
-    AutoTicks();
+  }
+  else if(prop=="xtickmode") {
+    std::string param(tolower(com.argin(2)->data));
+    if(param=="auto") {
+      MAKE_REF(xticklabelmode,Radio);
+      xticklabelmode.val("auto");
+    }
+  }
+  else if(prop=="ytickmode") {
+    std::string param(tolower(com.argin(2)->data));
+    if(param=="auto") {
+      MAKE_REF(yticklabelmode,Radio);
+      yticklabelmode.val("auto");
+    }
+  }
+  else if(prop=="ztickmode") {
+    std::string param(tolower(com.argin(2)->data));
+    if(param=="auto") {
+      MAKE_REF(zticklabelmode,Radio);
+      zticklabelmode.val("auto");
+    }
   }
 }
 
 void Axes::DrawOneAxis(axis_name ax,Matrix& tick,
-		       double* lim, double* limo,
-		       double ddo, bool log , bool logo)
+    double* lim, double* limo,
+    double ddo, bool log , bool logo, bool box)
 {
   MAKE_REF(ticklength,Matrix);
   MAKE_REF(gridlinestyle,Radio);
+  MAKE_REF(xcolor,ColorNone);
+  MAKE_REF(ycolor,ColorNone);
 
   for(int i=0;i<tick.len();i++) {
     if(tick(i)==lim[0]) continue;
     glLineWidth(1.0);
+
     glBegin(GL_LINES);
-
     switch (ax) {
-    case x_axis:
-      SET_COLOR(xcolor);
-      glVertex3d(LOGIT(tick(i),log),(limo[0]),grid_z);
-      glVertex3d(LOGIT(tick(i),log),(limo[0]+ddo*ticklength(0)),grid_z);
-      break;
-    case y_axis:
-      SET_COLOR(ycolor);
-      glVertex3d((limo[0]),LOGIT(tick(i),log),grid_z);
-      glVertex3d((limo[0]+ddo*ticklength(0)),LOGIT(tick(i),log),grid_z);
-      break;
+      case x_axis:
+        if(xcolor.IsColor()) {
+          SET_COLOR(xcolor);
+          glVertex3d(LOGIT(tick(i),log),(limo[0]),0);
+          glVertex3d(LOGIT(tick(i),log),(limo[0]+ddo*ticklength(0)),0);
+	  if(box) {
+            glVertex3d(LOGIT(tick(i),log),(limo[1]),0);
+            glVertex3d(LOGIT(tick(i),log),(limo[1]-ddo*ticklength(0)),0);
+	  }
+        }
+        break;
+      case y_axis:
+        if(ycolor.IsColor()) {
+          SET_COLOR(ycolor);
+          glVertex3d((limo[0]),LOGIT(tick(i),log),0);
+          glVertex3d((limo[0]+ddo*ticklength(0)),LOGIT(tick(i),log),0);
+          if(box) {
+            glVertex3d((limo[1]),LOGIT(tick(i),log),0);
+            glVertex3d((limo[1]-ddo*ticklength(0)),LOGIT(tick(i),log),0);
+          }
+        }
+        break;
     }
-
     glEnd();
 
+
+
     if(gridlinestyle()!="none") {
       glLineWidth(1.0);
       glEnable(GL_LINE_STIPPLE);
@@ -352,15 +362,20 @@ void Axes::DrawOneAxis(axis_name ax,Matrix& tick,
       glBegin(GL_LINES);
 
       switch (ax) {
-      case x_axis:
-	SET_COLOR(xcolor);
-	glVertex3d(LOGIT(tick(i),log),(limo[0]),grid_z);
-	glVertex3d(LOGIT(tick(i),log),(limo[1]),grid_z);
-	break;
-      case y_axis:
-	SET_COLOR(ycolor);
-	glVertex3d((limo[0]),LOGIT(tick(i),log),grid_z);
-	glVertex3d((limo[1]),LOGIT(tick(i),log),grid_z);
+        case x_axis:
+          if(xcolor.IsColor()) {
+            SET_COLOR(xcolor);
+            glVertex3d(LOGIT(tick(i),log),(limo[0]),0);
+            glVertex3d(LOGIT(tick(i),log),(limo[1]),0);
+          }
+          break;
+        case y_axis:
+          if(ycolor.IsColor()) {
+            SET_COLOR(ycolor);
+            glVertex3d((limo[0]),LOGIT(tick(i),log),0);
+            glVertex3d((limo[1]),LOGIT(tick(i),log),0);
+          }
+          break;
       }      
 
       glEnd();
@@ -368,7 +383,32 @@ void Axes::DrawOneAxis(axis_name ax,Matrix& tick,
       if(printing) gl2psDisable(GL2PS_LINE_STIPPLE);
     }      
   }
-  
+
+}
+
+void Axes::Reset()
+{
+  MAKE_REF(children,HandleVect);
+  children.Clear(); 
+
+  delete Properties["Box"];
+  COPY_DEFAULT(Box,Radio);
+
+  delete Properties["Legend"];
+  Legend* lgnd = new Legend(Self);
+  Properties["Legend"] = new HandleVect(lgnd->GetHandle());
+  Properties["Legend"]->readonly(true);
+
+  delete Properties["XLim"];
+  Properties["XLim"] = Real2Matrix(0,1);
+
+  delete Properties["YLim"];
+  Properties["YLim"] = Real2Matrix(0,1);
+
+  MAKE_REF(xlimmode,Radio);
+  MAKE_REF(ylimmode,Radio);
+  xlimmode.val("auto");
+  ylimmode.val("auto");
 }
 
 void Axes::SetupLimits()
@@ -399,12 +439,17 @@ void Axes::draw()
   MAKE_REF(children,HandleVect);
   MAKE_REF(xscale,Radio);
   MAKE_REF(yscale,Radio);
+  MAKE_REF(visible,Radio);
+  MAKE_REF(color,ColorNone);
+  MAKE_REF(xcolor,ColorNone);
+  MAKE_REF(ycolor,ColorNone);
+  MAKE_REF(box,Radio);
 
   GLdouble eqn[4] = {0,0,0,0};
   // Set figure local coord system
   glPushMatrix();
   glMatrixMode(GL_MODELVIEW);
-  glLoadIdentity();
+  //  glLoadIdentity();
 
   // Setup clipping planes
   eqn[0] = 1;
@@ -422,66 +467,87 @@ void Axes::draw()
   eqn[1] = -1;
   eqn[3] = (position(1)+position(3));
   glClipPlane (GL_CLIP_PLANE3, eqn);
- 
+
   // Draw Background
-  glPolygonMode(GL_FRONT,GL_FILL);
-  glPolygonMode(GL_BACK,GL_FILL);
-  glBegin(GL_POLYGON);
-  SET_COLOR(color);
-  glVertex2d( position(0),position(1) );
-  glVertex2d ( position(0)+position(2), position(1) ); 
-  glVertex2d ( position(0)+position(2), position(1)+position(3)); 
-  glVertex2d ( position(0), position(1)+position(3)); 
-  glEnd();
+  if(visible()=="on" && color.IsColor()) {
+    glPolygonMode(GL_FRONT,GL_FILL);
+    glPolygonMode(GL_BACK,GL_FILL);
+    glBegin(GL_POLYGON);
+    SET_COLOR(color);
+    glVertex3d( position(0),position(1),-dz );
+    glVertex3d ( position(0)+position(2), position(1) ,-dz); 
+    glVertex3d ( position(0)+position(2), position(1)+position(3),-dz); 
+    glVertex3d ( position(0), position(1)+position(3),-dz); 
+    glEnd();
+  }
 
   AutoScale();
   AutoTicks();
-  
+
   // Set axes local coord system
   glTranslated( position(0), position(1), 0);
   glScaled( position(2)/(xlimits[1]-xlimits[0]),
-	    position(3)/(ylimits[1]-ylimits[0]),
-	    1.0);
+      position(3)/(ylimits[1]-ylimits[0]),
+      1.0);
   glTranslated( -xlimits[0], -ylimits[0], 0);
 
-  // Draw Axes
-  glLineWidth(((Scalar*)GetProperty("LineWidth"))->GetVal());
-  glBegin(GL_LINES);
-  SET_COLOR(xcolor);
-  glVertex3d( xlimits[0] , ylimits[0],grid_z);
-  glVertex3d( xlimits[1] , ylimits[0],grid_z);
-  SET_COLOR(ycolor);
-  glVertex3d( xlimits[0] , ylimits[0],grid_z);
-  glVertex3d( xlimits[0] , ylimits[1],grid_z);
-  glEnd();
+  if(visible()=="on") {
+    // Draw Axes
+    glLineWidth(((Scalar*)GetProperty("LineWidth"))->GetVal());
+    glBegin(GL_LINES);
+    if(xcolor.IsColor()) {
+      SET_COLOR(xcolor);
+      glVertex3d( xlimits[0] , ylimits[0],0);
+      glVertex3d( xlimits[1] , ylimits[0],0);
+      if(box()=="on") {
+        glVertex3d( xlimits[0] , ylimits[1],0);
+        glVertex3d( xlimits[1] , ylimits[1],0);
+      }
+    }
+    if(ycolor.IsColor()) {
+      SET_COLOR(ycolor);
+      glVertex3d( xlimits[0] , ylimits[0],0);
+      glVertex3d( xlimits[0] , ylimits[1],0);
+      if(box()=="on") {
+        glVertex3d( xlimits[1] , ylimits[0],0);
+        glVertex3d( xlimits[1] , ylimits[1],0);
+      }
+    }
+    glEnd();
 
-  ocpl::Real dx,dy;
-  GetPixel(dx,dy);
-  // Draw Ticks and gridlines
-  MAKE_REF(xtick,Matrix);
-  MAKE_REF(ytick,Matrix);
-  DrawOneAxis(x_axis,xtick,xlimits,ylimits,dy,xscale()=="log",yscale()=="log");
-  DrawOneAxis(y_axis,ytick,ylimits,xlimits,dx,yscale()=="log",xscale()=="log");
+    ocpl::Real dx,dy;
+    GetPixel(dx,dy);
+    // Draw Ticks and gridlines
+    MAKE_REF(xtick,Matrix);
+    MAKE_REF(ytick,Matrix);
+    NEXT_LAYER
+      DrawOneAxis(x_axis,xtick,xlimits,ylimits,dy,xscale()=="log",yscale()=="log",box()=="on");
+    DrawOneAxis(y_axis,ytick,ylimits,xlimits,dx,yscale()=="log",xscale()=="log",box()=="on");
 
-  // All children
-  xticklabelhandle.AllDraw();
-  yticklabelhandle.AllDraw();
+    // All children
+    xticklabelhandle.AllDraw();
+    yticklabelhandle.AllDraw();
 
+    xlabel.AllDraw();
+    ylabel.AllDraw();
+    title.AllDraw();
+  }
 
-  xlabel.AllDraw();
-  ylabel.AllDraw();
-  title.AllDraw();
-
-  children.AllDraw();
+  // draw each child in it's own layer, back to front
+  for ( children.First() ; !children.IsDone() ; children.Next() ) {
+    NEXT_LAYER;
+    (::GetObjectD(children.CurrentHandle()))->draw();
+  }
 
+  NEXT_LAYER;
   legend.AllDraw();
- 
+
   glPopMatrix();
 }
 
 
 void Axes::GetCoords(const ocpl::Real screen_x, const ocpl::Real screen_y,
-		     ocpl::Real& axis_x, ocpl::Real& axis_y)
+    ocpl::Real& axis_x, ocpl::Real& axis_y)
 {
   MAKE_REF(position,Matrix);
   MAKE_REF(xscale,Radio);
@@ -489,7 +555,7 @@ void Axes::GetCoords(const ocpl::Real screen_x, const ocpl::Real screen_y,
 
   axis_x = (screen_x-position[0])/position[2]*(xlimits[1]-xlimits[0]) + xlimits[0];
   axis_y = (screen_y-position[1])/position[3]*(ylimits[1]-ylimits[0]) + ylimits[0];
-  
+
   if( xscale()=="log") axis_x = pow(10.0,axis_x);
   if( yscale()=="log") axis_y = pow(10.0,axis_y);
 }
@@ -508,61 +574,49 @@ void Axes::GetPixel(ocpl::Real& dx, ocpl::Real& dy)
 void Axes::SetLimits(ocpl::Real x1,ocpl::Real y1,ocpl::Real x2,ocpl::Real y2)
 {
   delete Properties["XLim"];
-  if(x1<x2) Properties["XLim"] = Real2Matrix(x1,x2);
-  else Properties["XLim"] = Real2Matrix(x2,x1);
-    
+  Properties["XLim"] = Real2Matrix(x1,x2);
+
   delete Properties["YLim"];
-  if(y1<y2) Properties["YLim"] = Real2Matrix(y1,y2);
-  else Properties["YLim"] = Real2Matrix(y2,y1);
+  Properties["YLim"] = Real2Matrix(y1,y2);
 
   MAKE_REF(xlimmode,Radio);
   MAKE_REF(ylimmode,Radio);
   xlimmode.val("manual");
   ylimmode.val("manual");
-  SetupLimits();
-  AutoTicks();
 }
 
 bool Axes::AutoScaleAxis(int na, double* lim)
 {
   MAKE_REF(children,HandleVect);
-  MAKE_REF(xscale,Radio);
-  MAKE_REF(yscale,Radio);
+  bool change = false;
+  MAKE_REF(xscale,Radio); 	 
+  MAKE_REF(yscale,Radio); 	 
 
   bool log = (na==0 ? (xscale()=="log") : (yscale()=="log") );
 
   children.First();
-  if(children.IsDone()) return false; // Empty List, don't change limits
-
-  Object* co = ::GetObjectD( children.CurrentHandle());
-  lim[0] = log ? co->lmin[na] : co->min[na];
-  lim[1] = log ? co->lmax[na] : co->max[na];
-
-  children.Next();
   while(!children.IsDone()) {
     Object* co = ::GetObjectD( children.CurrentHandle());
-    if(log) {
-      if(finite(co->lmin[na]) && finite(co->lmax[na])) {
-        if(co->lmin[na]<lim[0]) lim[0] = co->lmin[na];
-        if(co->lmax[na]>lim[1]) lim[1] = co->lmax[na];
-      }
-    }
+    if(log) { 	 
+      if(finite(co->lmin[na]) && finite(co->lmax[na])) { 	 
+        if((co->lmin[na]<lim[0]) || !change) lim[0] = co->lmin[na]; 	 
+        if((co->lmax[na]>lim[1]) || !change) lim[1] = co->lmax[na]; 	 
+        change = true; 	 
+      } 	 
+    } 	 
     else {
       if(finite(co->min[na]) && finite(co->max[na]) ) {
-        if(co->min[na]<lim[0]) lim[0] = co->min[na];
-        if(co->max[na]>lim[1]) lim[1] = co->max[na];
+        if((co->min[na]<lim[0]) || !change) lim[0] = co->min[na];
+        if((co->max[na]>lim[1]) || !change) lim[1] = co->max[na];
+        change = true;
       }
     }
     children.Next();
   }
 
-  double ti = PlotRound((lim[1]-lim[0]) / ((double) numticks-1));
-  if(fabs(ti)>1e-14) { 
-    if(!log) lim[0] = floor(lim[0] / ti)*ti;
-    //this hack ensures no rounding errors occur :( 
-    lim[1] = (ceil((lim[1]/ti) - 0.000001))*ti;
-  }
-  else { // limits are the same
+  if(!change) return false; // No child had any limit, so don't change graph's limit
+
+  if(lim[0] == lim[1]) {
     lim[0]-=0.5;
     lim[1]+=0.5;
   }
@@ -613,15 +667,27 @@ void Axes::AutoScale()
   SetupLimits();
 }
 
+// void Axes::AutoTicksAxis(Matrix& lim, Matrix& tick, bool log, int direction, double pixelSize)
+// {
+//   assert(direction==0 || direction==1);
+
+//   auto_tickers[direction]->SetRange(lim(0),lim(1));
+
+//   auto_tickers[direction]->SetPixelSize(pixelSize);
+//   auto_tickers[direction]->SetLogScale(log);
+
+//   auto_tickers[direction]->GetAutoTicks(tick);
+// }
+
 void Axes::AutoTicksAxis(Matrix& lim,Matrix& tick,bool log)
 {
-  double dt; 
+  double dt;
   double* t;
   int    nt;
- 
+
   if(!log) {
-    double fexp = floor(log10(lim(1)-lim(0)));
-    double sx = (lim(1)-lim(0))/pow(10.0, fexp)/10.0;   /* scaled x */
+    double fexp = floor(log10(fabs(lim(1)-lim(0))));
+    double sx = fabs(lim(1)-lim(0))/pow(10.0, fexp)/10.0;   /* scaled x */
     double rx = floor(sx);                    /* rounded x */
     double f = 10*(sx - rx);                  /* fraction between 0 and 10 */
 
@@ -629,26 +695,26 @@ void Axes::AutoTicksAxis(Matrix& lim,Matrix& tick,bool log)
     else if(f<=2) dt = 0.2;
     else if(f<=6) dt = 0.5;
     else          dt = 1.0;
-  
-    dt *= pow(10.0,fexp);
+
+    dt *= copysign(pow(10.0,fexp), lim(1)-lim(0));
     nt = static_cast<int>(floor((lim(1)-lim(0))/dt));
     double t0 = ceil(lim(0)/dt)*dt;
-    if( t0 + nt*dt <= lim(1)) nt++;
+    if( fabs(t0 + nt*dt) <= fabs(lim(1))) nt++;
 
     t = new double[nt];
     t[0] = t0;
     for(int i=1;i<nt;i++) {
       t[i] = t[i-1] + dt;
       if(fabs(t[i])<1e-14) t[i]=0;
-    }
+     }
   }
   else {
     double llim[2];
     llim[0] = log10(lim(0));
     llim[1] = log10(lim(1));
 
-    double fexp = floor(log10(llim[1]-llim[0]));
-    double sx = (llim[1]-llim[0])/pow(10.0, fexp)/10.0;   /* scaled x */
+    double fexp = floor(log10(fabs(llim[1]-llim[0])));
+    double sx = fabs(llim[1]-llim[0])/pow(10.0, fexp)/10.0;   /* scaled x */
     double rx = floor(sx);                    /* rounded x */
     double f = 10*(sx - rx);                  /* fraction between 0 and 10 */
 
@@ -656,13 +722,12 @@ void Axes::AutoTicksAxis(Matrix& lim,Matrix& tick,bool log)
     else if(f< 3) dt = 0.2;
     else if(f< 7) dt=0.5;
     else          dt = 1.0;
-  
-    dt *= pow(10.0,fexp);
+
+    dt *= copysign(pow(10.0,fexp), llim[1]-llim[0]);
     int dd = static_cast<int>(1.0/dt);
     nt = static_cast<int>(floor((llim[1]-llim[0])*dd))+1;
     double t0 = ceil(llim[0]);
 
-
     t = new double[nt];
     t[0] = pow(10.0,t0);
     int i;
@@ -670,7 +735,7 @@ void Axes::AutoTicksAxis(Matrix& lim,Matrix& tick,bool log)
       double tt0 = pow(10.0,t0+i/dd);
       if(i%dd ) t[i] = (i%dd)*dt*pow(10.0,1+floor(log10(tt0)));
       else t[i] = tt0;
-      
+
       // FIXME
       // bruteforce way, instead of calculating :/
       if(t[i]>lim(1)) break;
@@ -683,6 +748,8 @@ void Axes::AutoTicksAxis(Matrix& lim,Matrix& tick,bool log)
   delete tempt;
 }
 
+
+
 //! formats tick positions into a "a|b|c|d" string
 void Axes::TickLabels(Matrix& tick, String& label)
 {
@@ -710,7 +777,7 @@ std::string Axes::GetTickLabel(String& labels, int pos)
     else
       startpos++;
   };
-  
+
   std::string::size_type endpos = tmplab.find('|',startpos);
   if(endpos==std::string::npos)
     endpos = tmplab.length();
@@ -744,7 +811,10 @@ void Axes::AutoTicks()
   MAKE_REF(xtickmode,Radio);
   MAKE_REF(ytickmode,Radio);
   MAKE_REF(ztickmode,Radio);
-  
+  MAKE_REF(xaxislocation,Radio);
+  MAKE_REF(yaxislocation,Radio);
+
+
   xticklabelhandle.Clear();
   yticklabelhandle.Clear();
   zticklabelhandle.Clear();
@@ -754,11 +824,14 @@ void Axes::AutoTicks()
   GetPixel(dx,dy);
 
   if(xtickmode()=="auto")
+    //    AutoTicksAxis(xlim,xtick,xscale()=="log",0,dx);
     AutoTicksAxis(xlim,xtick,xscale()=="log");
   if(ytickmode()=="auto")
+    //    AutoTicksAxis(ylim,ytick,yscale()=="log",1,dy);
     AutoTicksAxis(ylim,ytick,yscale()=="log");
-  if(ztickmode()=="auto")
-    AutoTicksAxis(zlim,ztick,false);
+  // UNIMPLEMENTED
+  // if(ztickmode()=="auto")
+  //   AutoTicksAxis(zlim,ztick,zscale()=="log",2,dz)
 
   if(xticklabelmode()=="auto")
     TickLabels(xtick,xticklabel);
@@ -767,31 +840,58 @@ void Axes::AutoTicks()
 
   Text *txt;
   tl=-3*dx;
-  double xp = xscale()=="log" ? xlim(0)*pow(10.0,tl) : xlim(0)+tl;
+  double xp;
+  std::string alg;
+  if(yaxislocation()=="left") {
+    xp = xscale()=="log" ? xlim(0)*pow(10.0,tl) : xlim(0)+tl;
+    alg = "right";
+  }
+  else {
+    xp = xscale()=="log" ? xlim(1)*pow(10.0,-tl) : xlim(1)-tl;
+    alg = "left";
+  }
+
   labd = 0;
   for(int i=0;i<ytick.len();i++) {
     label=GetTickLabel(yticklabel,i);
     txt = new Text(GetHandle(), label.c_str() , xp , ytick(i),object_z);
     if(printing) txt->SetPrinting(true);
     yticklabelhandle.Add(txt->GetHandle());
-    ::Set(txt->GetHandle(),"HorizontalAlignment","right");
+    ::Set(txt->GetHandle(),"HorizontalAlignment",alg.c_str());
     ::Set(txt->GetHandle(),"VerticalAlignment","middle");
     ::Set(txt->GetHandle(),"Clipping","off");
     if(labd<txt->w()) labd = txt->w();
   }
-  pos[0] = xscale()=="log" ? xp*pow(10.,-dx*(labd+2)) : xp - dx*(labd+2);
+
+  if(yaxislocation()=="left") {
+    pos[0] = xscale()=="log" ? xp*pow(10.,-dx*(labd+2)) : xp - dx*(labd+2);
+    alg = "bottom";
+  }
+  else {
+    pos[0] = xscale()=="log" ? xp*pow(10.,dx*(labd+2)) : xp + dx*(labd+2);
+    alg = "top";
+  }
   pos[1] = yscale()=="log" ? 
     pow(10,(log10(ylim(1))+log10(ylim(0)))/2.) 
     : 
     (ylim(1)+ylim(0))/2.0;
   pos[2] = object_z;
   ::Set(ylabel(),"Position",pos,3);
-  ::Set(ylabel(),"VerticalAlignment","bottom");
+  ::Set(ylabel(),"VerticalAlignment",alg.c_str());
   ::Set(ylabel(),"HorizontalAlignment","center");
 
 
   tl=-2*dy;
-  double yp = yscale()=="log" ? ylim(0)*pow(10.0,tl) : ylim(0)+tl;
+  double yp;
+  if(xaxislocation()=="bottom") {
+    yp = yscale()=="log" ? ylim(0)*pow(10.0,tl) : ylim(0)+tl;
+    alg = "top";
+  }
+  else {
+    yp = yscale()=="log" ? ylim(1)*pow(10.0,-tl) : ylim(1)-tl;
+    alg = "bottom";
+  }
+
   labd = 0; // label delta
   for(int i=0;i<xtick.len();i++) {
     label = GetTickLabel(xticklabel,i);
@@ -799,21 +899,28 @@ void Axes::AutoTicks()
     if(printing) txt->SetPrinting(true);
     xticklabelhandle.Add(txt->GetHandle());
     ::Set(txt->GetHandle(),"HorizontalAlignment","center");
-    ::Set(txt->GetHandle(),"VerticalAlignment","top");
+    ::Set(txt->GetHandle(),"VerticalAlignment",alg.c_str());
     ::Set(txt->GetHandle(),"Clipping","off");
     if(labd<txt->h()) labd = txt->h();
   }
-  
+
   pos[0] = xscale()=="log" ?
     pow(10,(log10(xlim(1))+log10(xlim(0)))/2.) 
     : 
     (xlim(1)+xlim(0))/2.0;
-  pos[1] = yscale()=="log" ? yp*pow(10.,-dy*(labd+2)) : yp - dy*(labd+2);
+  if(xaxislocation()=="bottom") {
+    pos[1] = yscale()=="log" ? yp*pow(10.,-dy*(labd+2)) : yp - dy*(labd+2);
+    alg = "top";
+  }
+  else {
+    pos[1] = yscale()=="log" ? yp*pow(10.,dy*(labd+2)) : yp + dy*(labd+2);
+    alg = "bottom";
+  }
   pos[2] = object_z;
   ::Set(xlabel(),"Position",pos,3);
-  ::Set(xlabel(),"VerticalAlignment","top");
+  ::Set(xlabel(),"VerticalAlignment",alg.c_str());
   ::Set(xlabel(),"HorizontalAlignment","center");
-  
+
   // title
   pos[0] = xscale()=="log" ? 
     pow(10,(log10(xlim(1))+log10(xlim(0)))/2.) 
@@ -825,7 +932,7 @@ void Axes::AutoTicks()
   ::Set(title(),"Position",pos,3);
   ::Set(title(),"VerticalAlignment","bottom");
   ::Set(title(),"HorizontalAlignment","center");
-  
+
 }
 
 ocpl::Real Axes::PlotRound(ocpl::Real x)
@@ -855,7 +962,7 @@ void Axes::SetPrinting(bool set)
   MAKE_REF(ylabel,HandleVect);
   MAKE_REF(title,HandleVect);
   MAKE_REF(legend,HandleVect);
-  
+
   xticklabelhandle.SetPrinting(set);
   yticklabelhandle.SetPrinting(set);
   zticklabelhandle.SetPrinting(set);
@@ -885,10 +992,9 @@ void Axes::pop_zoombox()
     zoomstack.pop();
     SetLimits(zb.x1,zb.y1,zb.x2,zb.y2);
   }
+  // if we are aat the end of zoom do nothing
   else {
-    xlimmode.val("auto");
-    ylimmode.val("auto");
-    AutoScale();
-    AutoTicks();
+    // xlimmode.val("auto");
+    // ylimmode.val("auto");
   }
 }
