@@ -19,6 +19,7 @@
 #define MATHUTILS_H
 
 #include <cmath>
+#include <vector>
 #include "prop_matrix.h"
 
 void FindMinMax(Matrix& M,
@@ -30,5 +31,6 @@ void FindMinMax(Matrix& M,
 
 void InterpColor(Matrix& Colormap, Matrix& Clim, double z, double cv[3]);
 
+vector<unsigned int>* FactorList(unsigned int n);
 
 #endif
