@@ -17,7 +17,7 @@
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {@var{h}} = patch (@var{x},@var{y},@var{args})
-## This function adds a new patch pbject with vertexes (@var{x},@var{y}).
+## This function adds a new patch object with vertexes (@var{x},@var{y}).
 ##
 ## The remainder of the arguments, if any, are taken as property value pairs
 ## and are applied to the patch
