@@ -20,12 +20,15 @@
 
 class Root;
 
+#include <set>
+
 #include "globals.h"
 #include "object.h"
 #include "figurewindow.h"
 #include "figure.h"
 #include "op_com.h"
 
+
 //! Root object
 
 //! The Root object is the parent of all figures. It also hols all the
@@ -35,13 +38,14 @@ public:
   Root(ocpl::Handle Parent);
   ~Root() {};
   void AddFigure();
+  void DelWindow(FigureWindow* fw);
   //! deleting root will exit octplot so this is done in octplotapp
   virtual void DeleteMe() {}; 
   virtual void DeleteChild(ocpl::Handle Child);
   virtual void Parse(ocpl::command& com);
   virtual bool PreSet(ocpl::command& com);
-private:
-  int _last_fig;
+ private:
+  set<int> free_nums;
 };
 
 #endif
